@@ -30,10 +30,12 @@
 ;;; Code:
 (provide 'navi2ch-multibbs)
 (defconst navi2ch-multibbs-ident
-  "$Id: navi2ch-multibbs.el,v 1.35 2004/09/12 03:55:25 mami Exp $")
+  "$Id: navi2ch-multibbs.el,v 1.43 2006/07/03 22:23:55 knok Exp $")
 
+(eval-when-compile (require 'cl))
 (require 'navi2ch-http-date)
 (require 'navi2ch)
+(require 'navi2ch-be2ch)
 
 (defvar navi2ch-multibbs-func-alist nil
   "BBS $B$N<oN`$H4X?t72$N(B alist$B!#(B
@@ -50,6 +52,7 @@ FUNC-ALIST $B$O0J2<$NDL$j(B
  (url-to-board		. URL-TO-BOARD-FUNC)
  (url-to-article	. URL-TO-ARTICLE-FUNC)
  (send-message		. SEND-MESSAGE-FUNC)
+ (extract-post          . EXTRACT-POST-FUNC)
  (send-success-p	. SEND-MESSAGE-SUCCESS-P-FUNC)
  (error-string		. ERROR-STRING-FUNC)
  (board-update		. BOARD-UPDATE-FUNC)
@@ -59,7 +62,7 @@ BBS-P-FUNC(URI):
     URI $B$,$=$N(B BBS $B$N$b$N$J$i$P(B non-nil $B$rJV$9!#(B
 
 SUBJECT-CALLBACK-FUNC():
-    subject.txt $B$r<hF@$9$k$H$-$K(B navi2ch-net-update-file $B$G;H$o$l$k%3!<(B
+    subject.txt $B$r<hF@$9$k$H$-$K(B `navi2ch-net-update-file' $B$G;H$o$l$k%3!<(B
     $B%k%P%C%/4X?t(B
 
 ARTICLE-UPDATE-FUNC(BOARD ARTICLE START):
@@ -77,9 +80,19 @@ URL-TO-ARTICLE-FUNC(URL):
 URL $B$+$i(B article $B$KJQ49$9$k!#(B
 
 SEND-MESSAGE-FUNC(FROM MAIL MESSAGE
-		  SUBJECT BBS KEY TIME BOARD ARTICLE):
+		  SUBJECT BBS KEY TIME BOARD ARTICLE POST):
     MESSAGE $B$rAw?.$9$k!#(B
 
+EXTRACT-POST-FUNC(OLD-POST BUFFER):
+    MESSAGE $B$N:FAw?.$K;H$&>pJs$r<h$j=P$9!#<h$j=P$7$?%G!<%?$r(B
+    $BJV$jCM$H$7$FJV$9$H!":FAw$N$?$a$K(BSEND-MESSAGE-FUNC$B$r8F$S=P$9(B
+    $B$H$-$K!"$=$NCM$,(BPOST$B0z?t$KB+G{$5$l$^$9!#(B
+     
+    BUFFER$B$K$O!"(BSEND-MESSAGE-FUNC$B$,JV$7$?(BPROC$B$+$i<h$j=P$7$?%3%s(B
+    $B%F%s%D$r%G%3!<%I$7$?$b$N$,A^F~$5$l$F$$$^$9!#:FAw$r7+$jJV$9(B
+    $B>l9g!":G8e$K(BEXTRACT-POST-FUNC$B$,JV$7$?CM$,(BOLD-POST$B$KB+G{$5$l(B
+    $B$F$$$^$9!#(B
+
 SEND-MESSAGE-SUCCESS-P-FUNC(PROC):
     PROC $B$NAw?.%;%C%7%g%s$,@.8y$7$F$$$l$P(B non-nil $B$r!"(B
     $B<:GT$7$?$i(B nil $B$r!":F;n9T2DG=$J<:GT$J$i(B 'retry $B$rJV$9!#(B
@@ -92,8 +105,7 @@ BOARD-UPDATE-FUNC(BOARD):
 
 BOARD-GET-FILE-NAME-FUNC(BOARD &optional FILE-NAME)
     BOARD $B$N>pJs$rJ]B8$9$k%G%#%l%/%H%j$r4p=`$H$7$F!"(BFILE-NAME $B$N(B
-    $B@dBP%Q%9$rJV$9!#(B
-")
+    $B@dBP%Q%9$rJV$9!#(B")
 
 (defvar navi2ch-multibbs-variable-alist nil
   "BBS $B$N<oN`$HJQ?t72$N(B alist$B!#(B
@@ -106,8 +118,7 @@ VARIABLE-ALIST $B$O0J2<$NDL$j(B
 \((coding-system		. CODING-SYSTEM-VAR))
 
 CODING-SYSTEM-VAR:
-    $B$=$N(B BBS $B$N%U%!%$%k$NJ8;z%3!<%I(B
-")
+    $B$=$N(B BBS $B$N%U%!%$%k$NJ8;z%3!<%I(B")
 
 
 (defun navi2ch-multibbs-get-bbstype-subr (uri list)
@@ -137,7 +148,7 @@ CODING-SYSTEM-VAR:
    'subject-callback 'navi2ch-2ch-subject-callback))
 
 (defmacro navi2ch-multibbs-defcallback (name spec &rest body)
-  "navi2ch-net-update-file $B$KEO$9(B callback $B$rDj5A$9$k!#(B
+  "`navi2ch-net-update-file' $B$KEO$9(B callback $B$rDj5A$9$k!#(B
 SPEC $B$O(B (BBSTYPE [ARG]...)$B!#(B
 $B<B:]$K$O!"(Bcallback $B$rDj5A$9$k$N$KI,MW$J(B BBSTYPE $B$JHD$N(B coding-system
 $B$K$h$k(B decode, encode $B=hM}$r(B BODY $B$rI>2A$9$kA08e$K9T$J$&!"(BNAME $B$H$$$&(B
@@ -251,17 +262,27 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 	  result)
       (navi2ch-board-save-spid board spid))))
 
+(defun navi2ch-multibbs-send-message-error-string (board proc)
+  (let* ((func (navi2ch-multibbs-get-func
+		(navi2ch-multibbs-get-bbstype board)
+		'error-string
+		'navi2ch-2ch-send-message-error-string))
+	 (err (funcall func proc)))
+    (or err
+	(let ((status (and proc (navi2ch-net-get-status proc))))
+	  (when status
+	    (concat "HTTP status: " status))))))
+
 (defun navi2ch-multibbs-send-message
   (from mail message subject board article)
   (let* ((bbstype      (navi2ch-multibbs-get-bbstype board))
 	 (send         (navi2ch-multibbs-get-func
 			bbstype 'send-message 'navi2ch-2ch-send-message))
+	 (extract-post (navi2ch-multibbs-get-func
+			bbstype 'extract-post 'navi2ch-2ch-extract-post))
 	 (success-p    (navi2ch-multibbs-get-func
 			bbstype 'send-success-p
 			'navi2ch-2ch-send-message-success-p))
-	 (error-string (navi2ch-multibbs-get-func
-			bbstype 'error-string
-			'navi2ch-2ch-send-message-error-string))
 	 (bbs          (let ((uri (navi2ch-board-get-uri board)))
 			 (string-match "\\([^/]+\\)/$" uri)
 			 (match-string 1 uri)))
@@ -279,10 +300,11 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 					    navi2ch-net-http-proxy-password))
 	 (tries 2)			; $BAw?.;n9T$N:GBg2s?t(B
 	 (message-str "send message...")
-	 (result 'retry))
+	 (result 'retry)
+	 (post-data nil))
     (dotimes (i tries)
       (let ((proc (funcall send from mail message subject bbs key time
-			   board article)))
+			   board article post-data)))
 	(message message-str)
 	(setq result (funcall success-p proc))
 	(cond ((eq result 'retry)
@@ -290,7 +312,8 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 		 (with-temp-buffer
 		   (insert (decode-coding-string
 			    (navi2ch-net-get-content proc)
-			    navi2ch-coding-system))
+			    (navi2ch-board-get-coding-system board)))
+		   (setq post-data (funcall extract-post post-data (current-buffer)))
 		   (navi2ch-replace-html-tag-with-buffer)
 		   (goto-char (point-min))
 		   (while (re-search-forward "[ \t]*\n\\([ \t]*\n\\)*" nil t)
@@ -305,7 +328,7 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 	       (message (concat message-str "succeed"))
 	       (return result))
 	      (t
-	       (let ((err (funcall error-string proc)))
+	       (let ((err (navi2ch-multibbs-send-message-error-string board proc)))
 		 (if (stringp err)
 		     (message (concat message-str "failed: %s") err)
 		   (message (concat message-str "failed"))))
@@ -409,8 +432,10 @@ START $B$,(B non-nil $B$J$i$P%l%9HV9f(B START $B$+$i$N:9J,$r<hF@$9$k!#(B
 	  (setq list (cons (cons 'kako kako) list)))
 	list))))
 
+(defvar navi2ch-2ch-send-message-last-board nil)
+
 (defun navi2ch-2ch-send-message
-  (from mail message subject bbs key time board article)
+  (from mail message subject bbs key time board article &optional post)
   (let ((url         (navi2ch-board-get-bbscgi-url board))
 	(referer     (navi2ch-board-get-uri board))
 	(spid        (navi2ch-board-load-spid board))
@@ -424,6 +449,12 @@ START $B$,(B non-nil $B$J$i$P%l%9HV9f(B START $B$+$i$N:9J,$r<hF@$9$k!#(B
 		      (if subject
 			  (cons "subject" subject)
 			(cons "key"    key)))))
+
+    (let ((hanamogera-cookie (cdr (assq 'hanamogera-cookie post))))
+      (when hanamogera-cookie
+	(setq param-alist (cons hanamogera-cookie param-alist))))
+
+    (setq navi2ch-2ch-send-message-last-board board)
     (setq spid
 	  (when (and (consp spid)
 		     (navi2ch-compare-times (cdr spid) (current-time)))
@@ -434,9 +465,13 @@ START $B$,(B non-nil $B$J$i$P%l%9HV9f(B START $B$+$i$N:9J,$r<hF@$9$k!#(B
 	    (list (cons "Content-Type" "application/x-www-form-urlencoded")
 		  (cons "Cookie" (concat "NAME=" from "; MAIL=" mail
 					 (if spid (concat "; SPID=" spid
-							  "; PON=" spid))))
+							  "; PON=" spid))
+					 (if (navi2ch-be2ch-login-p)
+					     (concat "; MDMD=" navi2ch-be2ch-mdmd
+						     "; DMDM=" navi2ch-be2ch-dmdm))))
 		  (cons "Referer" referer))
-	    (navi2ch-net-get-param-string param-alist))))
+	    (navi2ch-net-get-param-string param-alist
+					  (navi2ch-board-get-coding-system board)))))
       (setq spid (navi2ch-net-send-message-get-spid proc))
       (if spid (navi2ch-board-save-spid board spid))
       proc)))
@@ -457,10 +492,17 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 	      start (and (or start end) "-") end
 	      (and nofirst "n")))))
 
-(defalias 'navi2ch-2ch-send-message-success-p
-  'navi2ch-net-send-message-success-p)
-(defalias 'navi2ch-2ch-send-message-error-string
-  'navi2ch-net-send-message-error-string)
+(defun navi2ch-2ch-send-message-success-p (proc)
+  (navi2ch-net-send-message-success-p
+   proc
+   (navi2ch-board-get-coding-system
+    navi2ch-2ch-send-message-last-board)))
+
+(defun navi2ch-2ch-send-message-error-string (proc)
+  (navi2ch-net-send-message-error-string
+   proc
+   (navi2ch-board-get-coding-system
+    navi2ch-2ch-send-message-last-board)))
 
 (defun navi2ch-2ch-board-update (board)
   (let ((file (navi2ch-board-get-file-name board))
@@ -486,4 +528,22 @@ START, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
 				   navi2ch-board-subject-file-name)
 			       (match-string 1 uri)))))))
 
+(defun navi2ch-2ch-extract-post (old-post buffer)
+  ;; Get hana and mogera from following string.
+  ;; <input type=hidden name="hana" value="mogera">  
+  (with-current-buffer buffer
+    (save-excursion
+      (save-match-data
+	(goto-char (point-max))
+	(when (re-search-backward 
+	       "<input[ \t]+type=hidden[ \t]+name=\"\\([^\"]+\\)\"[ \t]+value=\"\\([^\"]+\\)\""
+	       nil
+	       t)
+	  ;; (('hana-mogera-cookie "hana" . "mogera"))
+	  (list 
+	   (cons 'hanamogera-cookie (cons 
+				      (match-string 1) ; hana
+				      (match-string 2) ; mogera
+				      ))))))))
+
 ;;; navi2ch-multibbs.el ends here
